# coding: utf-8
from os.path import isdir, isfile, join
from os import makedirs, listdir, rename, unlink
from shutil import rmtree
from zipfile import ZipFile
from json import dumps
import traceback
from pyeole.lock import acquire, release, is_locked

from eoleaaf.parseaaf import main as complet_main
from eoleaaf.parsedelta import main as delta_main
from eoleaaf.config import path_aaf_complet
from eoleaaf.util import db
from logging import getLogger

from .action import describe as _describe, submit as _submit, _get_tiramisu_informations

aaflog = getLogger("AAF")


def _send_error_message(msg, remove_lock=True):
    return _send_message(u"Erreur {}".format(msg), 'error', remove_lock)


def _send_message(msg, level='info', remove_lock=True):
    if remove_lock:
        try:
            release('importaaf', level='system')
        except:
            aaflog.error(msg)
            traceback.print_exc()
            pass
    return dumps({'message': {'type': level, 'text': msg}})


def processfile(filename, level, delete=False, **kwargs):
    """
    :param filename: fullpath of the archive to unzip
    :param level: type of import ('complète' or 'delta')
    :param delete: if True, remove the import file after successfull process
    """
    aaflog.info(20*'*')
    aaflog.info('*** Début du traitement des fichiers AAF (importation {1}) : {0}'.format(filename, level))
    if level == 'complète':
        main = complet_main
    elif level == 'delta':
        main = delta_main
    else:
        return _send_error_message("paramètre invalide : type d'import", False)
    try:
        acquire('importaaf', level="system")
    except Exception as err:
        return _send_error_message("impossible de lancer l'import complet : {}".format(err), False)

    # the file must exists
    if not isfile(filename):
        return _send_error_message("impossible de trouver le fichier à décompresser : {}".format(filename))

    try:
        kwargs = {}
        if db.db.user.find_one():
            if level == 'complète':
                kwargs['maj_mode'] = True
        else:
            if level == 'delta':
                return _send_error_message("la base est vide, impossible de réaliser une importation de type delta")
            if level == 'complète':
                kwargs['maj_mode'] = False
    except Exception as err:
        traceback.print_exc()
        return _send_error_message("impossible de vérifier si la base est vide".format(err))

    # purge the folders
    try:
        if isdir(path_aaf_complet):
            rmtree(path_aaf_complet)
        makedirs(path_aaf_complet)
    except Exception as err:
        traceback.print_exc()
        return _send_error_message("à la création du répertoire de travail : {}".format(err))

    # unzip the files
    try:
        zip_ref = ZipFile(filename, 'r')
        zip_ref.extractall(path_aaf_complet)
        zip_ref.close()
        # move the files in path_aaf_complet if there is any subfolder
        for fname in listdir(path_aaf_complet):
            fullname = join(path_aaf_complet, fname)
            if isdir(fullname):
                for subfname in listdir(fullname):
                    rename(join(fullname, subfname), join(path_aaf_complet, subfname))
    except Exception as err:
        traceback.print_exc()
        return _send_error_message("à la décompression des fichiers : {}".format(err))

    try:
        # imports the files in mongodb
        main(path_aaf_complet, **kwargs)
    except Exception as err:
        traceback.print_exc()
        return _send_error_message("à l'importation des comptes dans la base : {}".format(err))

    # Delete AAF zip file if success
    if delete:
        unlink(filename)
        aaflog.info('Suppression du fichier {0}'.format(filename))

    aaflog.info('*** Fin du traitement des fichiers AAF')
    aaflog.info(20*'*')
    return _send_message('Importation effectuée')


def describe(*args, **kwargs):
    if is_locked(name='importaaf', level='system'):
        return _send_error_message("un import AAF est en cours", False)
    return _describe(*args, **kwargs)


def submit(*args, **kwargs):
    if is_locked(name='importaaf', level='system'):
        return _send_error_message("une action AAF est en cours", False)
    dico, values, config = _get_tiramisu_informations(args)
    upload_folder = config.creole.services.ead3_upload_path
    kwargs['dico'] = (dico, values, config)
    if dico['importaaf.aaf.aaf_filename'] == "Aucun fichier téléversé depuis l'action 'Gérer les fichiers'":
        return _send_error_message('aucun fichier selectionné', False)
    filename = join(upload_folder, dico['importaaf.aaf.aaf_filename'])
    return processfile(filename, dico['importaaf.aaf.aaf_type'], delete=True)


__CUSTOM_MODULES__ = ('processfile',)
